import os, json, yaml, logging

from enums import Severity, Status
from k8s_tasks_core import TaskLogger, ControlPlane
from tasks_core import get_postgres_connection
from k8s_tasks_core import StdErrException


def install_mongodb_k8s_standalone_task(data, task_id):
    task_name = "install mongo stand alone task"

    try:
        data = json.loads(data)
        created_by = int(data["created_by"])

        node_data = json.loads(data["node_data"])
        host_data = json.loads(data["host_data"])
        hostname = host_data["name"]

        mongo_user = node_data["database_user"]
        mongo_password = node_data["database_pwd"]
        mongo_port = int(node_data["port"])

        # placeholders specific to mongodb
        path = f"/123cluster/mongodb/{hostname}:{mongo_port}"
        conf_dir = "/root/config/"
        template = "/root/config/values-mongodb.yaml"

        # placeholders specific to k8s
        # namespace = 'mongodb'

    except Exception as error:
        logging.error(f"{task_name}: {str(error)}")
        return

    logger = TaskLogger(task_id, task_name, created_by)
    cp = ControlPlane(host_data)

    with logger.step("initializing task") as step:
        pass

    # if not k8s.pv_exists(pv_name):
    #     with logger.step('applying persistent volume') as step:
    #         host.execute_command(f'kubectl apply -f "/root/{pv_name_file}.yaml"')

    with logger.step("changing configuration"):
        os.makedirs(conf_dir, exist_ok=True)

        with open(template, "r") as file:
            confs = list(yaml.safe_load_all(file))
            for conf in confs:
                # conf['metadata']['namespace'] = f'{namespace}'

                if conf["kind"] == "Secret":
                    conf["stringData"]["mongo-root-username"] = f"{mongo_user}"
                    conf["stringData"]["mongo-root-password"] = f"{mongo_password}"

                if conf["kind"] == "Service":
                    for port in conf["spec"]["ports"]:
                        port["port"] = mongo_port
                        port["targetPort"] = mongo_port

        with open(f"{conf_dir}/values-mongodb.yaml", "w") as file:
            yaml.safe_dump_all(confs, file, default_flow_style=False)

    with logger.step("installing mongodb") as step:
        with open(f"{conf_dir}/values-mongodb.yaml", "r") as file:
            installation_confs = file.read()

        cp.execute_command(f"kubectl apply -f - <<EOF\n{installation_confs}\nEOF")

    with logger.step("task finished") as step:
        pass


def uninstall_mongodb_k8s_standalone_task(data, task_id):
    task_name = "uninstall mongodb standalone task"

    try:
        data = json.loads(data)
        created_by = int(data["created_by"])
        host_data = data["host_data"]
        node_data = json.loads(data["node_data"])
        hostname = host_data["name"]

        # placeholders specific to mongodb
        port = "27017"
        path = f"/123cluster/mongodb/{hostname}:{port}"
        template = "/root/config/values-mongodb.yaml"

        # placeholders specific to helm chart
        release_name = "mongodb"
        svc_name = "mongodb-external"
        secrete_name = "mongodb-secret"

        # placeholders specific to k8s
        # namespace = 'mongodb'
        pv_name = "mongodb-pv"
        pvc_name = "mongodb-data-mongodb-0"
    except Exception as error:
        logging.error(f"{task_name}: {str(error)}")
        return

    logger = TaskLogger(task_id, task_name, created_by)
    cp = ControlPlane(host_data)

    with logger.step("initializing task") as step:
        pass

    with logger.step("check statefulset ") as step:
        if cp.is_statefulset_exists(release_name):
            with logger.step("delete mongo statefulset") as step:
                cp.execute_command(f"kubectl delete statefulset {release_name}")

    with logger.step("check svc") as step:
        if cp.is_svc_exists(svc_name):
            with logger.step("delete mongo svc") as step:
                cp.execute_command(f"kubectl delete svc {svc_name}")

    with logger.step("check secret") as step:
        if cp.is_secret_exists(secrete_name):
            with logger.step("delete mongo secret") as step:
                cp.execute_command(f"kubectl delete secret {secrete_name}")

    with logger.step("check pvc") as step:
        if cp.is_pvc_exists(pvc_name):
            with logger.step("delete mongo persistent volume claim") as step:
                cp.execute_command(f"kubectl delete pvc {pvc_name}")

    with logger.step("check pv") as step:
        if cp.is_pv_exists(pv_name):
            with logger.step("delete mongo persistent volume") as step:
                cp.execute_command(f"kubectl delete pv {pv_name}")

    # with logger.step('uninstall mongodb') as step:
    #     host.execute_command(
    #         f'kubectl delete -f {template}')

    with logger.step("task finished") as step:
        with get_postgres_connection() as conn:
            with conn.cursor() as cur:
                cur.execute(
                    """UPDATE "clusterCore_c123_node" SET severity=%s, status=%s, ind_started='false',
                 ind_deleted='true' WHERE id=%s""",
                    (
                        Severity.NOT_OK.value,
                        Status.DELETED.value,
                        node_data["id"],
                    ),
                )


def mongodump_task(data, task_id):
    task_name = "dump mongo task"

    try:
        data = json.loads(data)
        created_by = int(data["created_by"])
        host_data = data["host_data"]
        dump = json.loads(data["dump_data"])
        node_data = json.loads(data["node_data"])
        hostname = host_data["name"]
        db_name = data.get("db_name")
        mongo_user = node_data["database_user"]
        mongo_password = node_data["database_pwd"]
        if db_name:
            db_id = data.get("db_id")

        # placeholders specific to mongodb
        port = "27017"
        path = f"/123cluster/mongodb/{hostname}:{port}"
        dump_path = "/123cluster-test/mongodb/168.119.125.84:27017/data/dump/"

        # placeholders specific to helm chart
        release_name = "mongodb"
        svc_name_1 = "mongodb"
        svc_name_2 = "mongodb-external"
        secrete_name = "mongodb-secret"

        # placeholders specific to k8s
        # namespace = 'mongodb'
        # pod_name = find_pods_name(data=host_data, namespace=namespace, target_pod_name=release_name)['found_pod'][0]
        pod_name = "mongodb-0"
    except Exception as error:
        logging.error(f"{task_name}: {str(error)}")
        return

    logger = TaskLogger(task_id, task_name, created_by)
    cp = ControlPlane(host_data)

    with logger.step("initializing task") as step:
        pass

    with logger.step("checking if any pod exists in stateful set") as step:
        # if not k8s.dpl_pod_exists(dpl_name, namespace):
        #     step.status = Severity.NOT_OK.value
        #     return
        pass

    with logger.step("making mongo dumpall") as step:
        try:
            cp.execute_command(
                f"kubectl exec {pod_name} -- mongodump --host=localhost --port=27017 "
                f"-u {mongo_user} -p {mongo_password} --authenticationDatabase admin --out={dump_path}"
            )
        except StdErrException as exception:
            if not "done dumping" in exception.stderr:
                print("debug: dump havent done")
                raise

    with logger.step("checking the dump size") as step:
        cmd_streams = cp.execute_command(f"sudo du -b '{dump_path}' | awk '{{print $1}}'")
        size = cmd_streams.strip()
        backup_size = float(size) / 1024
        if backup_size < 0.01:
            backup_size = 0.01

    with logger.step("Getting dump modified date") as step:
        command = f"""modified_time=$(($(sudo stat -c %Y '{dump_path}') + 1))
        formatted_date=$(date -d @$modified_time +%Y-%m-%d\ %H:%M:%S)
        echo $formatted_date"""
        cmd_streams = cp.execute_command(command=command)
        modified_date = cmd_streams

    with logger.step("Updating dump info") as step:
        with get_postgres_connection() as conn:
            with conn.cursor() as cur:
                cur.execute(
                    """UPDATE "clusterCore_c123_inventory" SET ex_params = ex_params || '{"modified_date":\"%s\"}', status='true' WHERE id=%s"""
                    % (modified_date, dump["id"])
                )
                conn.commit()
                if db_name is None:
                    cur.execute(
                        """UPDATE "clusterCore_c123_node" SET severity=%s, status=%s, ind_started='true' WHERE id=%s""",
                        (
                            Severity.OK.value,
                            Status.OK.value,
                            node_data["id"],
                        ),
                    )
                    conn.commit()
                else:
                    cur.execute(
                        """UPDATE "clusterCore_c123_database" SET severity=%s, status=%s, ind_started='true' WHERE id=%s""",
                        (
                            Severity.OK.value,
                            Status.OK.value,
                            db_id,
                        ),
                    )
                    conn.commit()

                cur.execute(
                    """UPDATE "clusterCore_c123_inventory" SET ex_params = jsonb_set(ex_params, '{{size}}', '{0}', true), status='true' WHERE id={1}""".format(
                        backup_size, dump["id"]
                    )
                )
                conn.commit()
                cur.execute(
                    """UPDATE "clusterCore_c123_inventory" SET ex_params = jsonb_set(ex_params, '{{backup_path}}', '"{0}"', true) WHERE id={1}""".format(
                        dump_path, dump["id"]
                    )
                )
                conn.commit()

    with logger.step("task finished") as step:
        pass


def mongorestore_task(data, task_id):
    task_name = "restore mongo task"

    try:
        data = json.loads(data)
        created_by = int(data["created_by"])
        host_data = data["host_data"]
        dump = json.loads(data["dump"])
        node_data = json.loads(data["node_data"])
        hostname = host_data["name"]
        db_name = data.get("db_name")
        mongo_user = node_data["database_user"]
        mongo_password = node_data["database_pwd"]
        if db_name:
            db_id = data.get("db_id")

        # placeholders specific to mongodb
        port = "27017"
        path = f"/123cluster/mongodb/{hostname}:{port}"
        dump_path = "/123cluster-test/mongodb/168.119.125.84:27017/data/dump/"

        # placeholders specific to helm chart
        # release_name = json.loads(data['release_name'])
        # chart_name = json.loads(data['chart_name'])
        # release_version = json.loads(data['chart_version'])

        # placeholders specific to k8s
        # namespace = json.loads(data['namespace'])
        # dpl_name = json.loads(data['release_name'])
        # storageclass_name_file = json.loads(data['storageclass_name_file'])
        # storageclass_name = json.loads(data['storageclass_name'])
        # pv_name = json.loads(data['pv_name'])
        # pv_name_file = json.loads(data['pv_name_file'])
        # pod_name = find_pods_name(data=host_data, namespace=namespace, target_pod_name=release_name)['found_pod'][0]
        pod_name = "mongodb-0"
    except Exception as error:
        logging.error(f"{task_name}: {str(error)}")
        return

    logger = TaskLogger(task_id, task_name, created_by)
    cp = ControlPlane(host_data)

    with logger.step("initializing task") as step:
        pass

    # with logger.step('checking if any pod exists in stateful set') as step:
    #     if not k8s.dpl_pod_exists(dpl_name, namespace):
    #         step.status = Severity.NOT_OK.value
    #         return

    with logger.step("making mongo restore") as step:
        try:
            cp.execute_command(
                f"kubectl exec {pod_name} -- mongorestore --host=localhost --port=27017 "
                f"-u {mongo_user} -p {mongo_password} --authenticationDatabase admin --dir={dump_path} --drop"
            )
        except StdErrException as exception:
            if not 0 == exception.excode:
                print("debug: dump havent done")
                raise

    # TODO: sudo du -sb '{node_data['dir_123Cluster']}' | awk '{{print $1}}'
    with logger.step("checking the dump size") as step:
        cmd_streams = cp.execute_command(f"sudo du -sb '{dump_path}' | awk '{{print $1}}'")
        node_size = float(cmd_streams.strip()) / 1024

    with logger.step("task finished") as step:
        with get_postgres_connection() as conn:
            with conn.cursor() as cur:
                cur.execute(
                    """UPDATE "clusterCore_c123_node" SET severity=%s, status=%s, size=%s, ind_started='true' WHERE id=%s""",
                    (
                        Severity.OK.value,
                        Status.OK.value,
                        node_size,
                        node_data["id"],
                    ),
                )
                # if db_name:
                #     cur.execute(
                #         """UPDATE "clusterCore_c123_database" SET severity=%s, status=%s, size=%s, ind_started='true', ind_deleted = 'false' WHERE id=%s""",
                #         (Severity.OK.value, Status.OK.value, db_size, db_id,))

                conn.commit()
